import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

import '../mixer/mix.dart';
import 'remix_widget.dart';

class Box extends RemixWidget {
  Box(
    Mix mix, {
    required this.child,
    Key? key,
  }) : super(mix, key: key);

  final Widget? child;

  EdgeInsetsGeometry? get _paddingIncludingDecoration {
    if (attr.decoration == null || attr.decoration!.value.padding == null) {
      return attr.padding?.value;
    }
    final decorationPadding = attr.decoration!.value.padding;
    if (attr.padding == null) return decorationPadding;
    return attr.padding!.value.add(decorationPadding!);
  }

  @override
  Widget build(BuildContext context) {
    var current = child;

    if (attr.display != null) {
      if (attr.display!.value == false) {
        return SizedBox.shrink();
      }
    }

    if (child == null &&
        (attr.constraints == null || !attr.constraints!.value.isTight)) {
      current = LimitedBox(
        maxWidth: 0.0,
        maxHeight: 0.0,
        child: ConstrainedBox(
          constraints: const BoxConstraints.expand(),
        ),
      );
    }

    if (attr.aspectRatio != null) {
      current = AspectRatio(
        aspectRatio: attr.aspectRatio!.value,
        child: current,
      );
    }

    if (attr.alignment != null) {
      current = Center();
      current = Align(
        alignment: attr.alignment!.value,
        child: current,
      );
    }

    final effectivePadding = _paddingIncludingDecoration;
    if (effectivePadding != null) {
      current = Padding(
        padding: effectivePadding,
        child: current,
      );
    }

    if (attr.color != null) {
      current = ColoredBox(
        color: attr.color!.value,
        child: current,
      );
    }

    if (attr.decoration != null) {
      current = DecoratedBox(
        decoration: attr.decoration!.value,
        child: current,
      );
    }

    if (attr.opacity != null) {
      current = Opacity(
        opacity: attr.opacity!.value,
        child: current,
      );
    }

    /// Set child constraints
    if (attr.constraints != null) {
      current = ConstrainedBox(
        constraints: attr.constraints!.value,
        child: current,
      );
    }

    if (attr.maxHeight != null || attr.maxWidth != null) {
      current = LimitedBox(
        maxHeight: attr.maxHeight?.value ?? double.infinity,
        maxWidth: attr.maxWidth?.value ?? double.infinity,
        child: current,
      );
    }

    if (attr.rotate != null) {
      current = RotatedBox(
        quarterTurns: attr.rotate!.value,
        child: current,
      );
    }

    if (attr.margin != null) {
      current = Padding(
        padding: attr.margin!.value,
        child: current,
      );
    }

    if (attr.flex != null || attr.flexFit != null) {
      current = Flexible(
        flex: attr.flex?.value ?? 1,
        fit: attr.flexFit?.value ?? FlexFit.loose,
        child: current!,
      );
    }

    return current!;
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties.add(
      DiagnosticsProperty<AlignmentGeometry>('alignment', attr.alignment?.value,
          showName: false, defaultValue: null),
    );
    properties.add(
      DiagnosticsProperty<EdgeInsetsGeometry>('padding', attr.padding?.value,
          defaultValue: null),
    );

    if (attr.color != null) {
      properties.add(
        DiagnosticsProperty<Color>('bg', attr.color?.value),
      );
    } else {
      properties.add(
        DiagnosticsProperty<Decoration>('bg', attr.decoration?.value,
            defaultValue: null),
      );
    }

    properties.add(
      DiagnosticsProperty<BoxConstraints>(
          'constraints', attr.constraints?.value,
          defaultValue: null),
    );
    properties.add(
      DiagnosticsProperty<EdgeInsetsGeometry>('margin', attr.margin?.value,
          defaultValue: null),
    );
  }
}
